using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Drawing;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Tools;

namespace ToolTipExtGrid
{
	#region GridTextExtender Class
	/// <summary>
	/// Provides a "tooltip" text extender for a cell in a Syncfusion grid if the
	/// cell's text is longer than the cell width. Use a separate GridTextExtender
	/// class for each grid.
	/// </summary>
	/// <example>
	///		// The the form class:
	///		private GridTextExtender m_oGridTextExtender = null;
	///		
	///		// On form load:
	///		m_oGridTextExtender = new GridTextExtender(this);
	///		
	///		// In the grid's MouseMove event handler:
	///		private void grid_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
	///		{
	///			m_oGridTextExtender.Show(grid, new Point(e.X, e.Y));
	///		}
	/// </example>
	public class GridTextExtender
	{
		private Control m_oParent = null;
		private int m_nHoverCol = -1;
		private int m_nHoverRow = -1;
		private Color m_BackColour = System.Drawing.SystemColors.Info;
		private Label m_oLabel = null;
		private Hashtable m_ExtendedHeaderText = null;
		private bool m_ShowHeaderToolTip = false;

		/// <summary>Return the last column over which the mouse hovered</summary>
		public int HoverCol
		{
			get { return m_nHoverCol; }
		}

		/// <summary>Return the last row over which the mouse hovered</summary>
		public int HoverRow
		{
			get { return m_nHoverRow; }
		}

		/// <summary>The background colour to use</summary>
		public Color BackColour
		{
			set { m_BackColour = value; }
			get { return m_BackColour; }
		}

		public bool ShowHeaderToolTip
		{
			set
			{
				m_ShowHeaderToolTip = value;		
				if( m_ExtendedHeaderText == null )
					m_ExtendedHeaderText = new Hashtable();
			}
			get { return m_ShowHeaderToolTip; }
		}

		public Hashtable ExtendedHeaderText
		{
			set{m_ExtendedHeaderText = value;}
			get { return m_ExtendedHeaderText; }
		}

		/// <summary>Return the extended text label object</summary>
		public Label TextExtender
		{
			get { return m_oLabel; }
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="oParent">The parent form</param>
		public GridTextExtender(Control oParent)
		{
			// Create tooltip like label
			m_oParent = oParent;
			m_oLabel = new Label();
			m_oLabel.BackColor = m_BackColour;
			m_oLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			m_oLabel.Name = "GridTextExtender";
			m_oLabel.TabIndex = 0;
			m_oLabel.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			m_oLabel.Visible = false;

			// Set up event handler so that the user can hide the label
			// by clicking it
			m_oLabel.Click += new System.EventHandler(Label_Click);

			// Add the label to the parents control list
			m_oParent.Controls.Add(m_oLabel);
		}

		/// <summary>
		/// Show the text extender if required, otherwise hide it
		/// </summary>
		/// <param name="Grid">The grid</param>
		/// <param name="oPoint">The current mouse position</param>
		public void Show(Syncfusion.Windows.Forms.Grid.GridControlBase oGrid, Point oPoint)
		{
			int nRow, nCol;

			// Check if we're inside the grid, but not over the same cell
			if (oGrid.PointToRowCol(oPoint, out nRow, out nCol) && 
				(nCol != m_nHoverCol || nRow != m_nHoverRow))
			{
				m_nHoverCol = nCol;
				m_nHoverRow = nRow;

				if (NeedsExtending(oGrid, nRow, nCol))
					ShowExtendedText(oGrid, nRow, nCol);
				else
					Hide();
			}
			else if (!oGrid.PointToRowCol(oPoint, out nRow, out nCol))
			{
				Hide();
				m_nHoverCol = -1;
				m_nHoverRow = -1;
			}
		}

		/// <summary>
		/// Hide the text extender
		/// </summary>
		public void Hide()
		{
			m_oLabel.Hide();
		}

		private bool NeedsExtending(Syncfusion.Windows.Forms.Grid.GridControlBase oGrid, int nRow, int nCol)
		{
			// We don't need (0,0) text extensions
			if (nRow == 0 && nCol == 0)
				return false;
			else if (oGrid.Model[nRow,nCol].CellType == "ColumnHeaderCell" )
				return ShowHeaderToolTip;
			// Compute the text width
			Graphics oGraphics = oGrid.CreateGraphics();
			GridStyleInfo oStyle = oGrid.Model[nRow, nCol];
			float fTextWidth = oGraphics.MeasureString(oStyle.Text, oStyle.GdipFont).Width;
			int nMarginSpace = oStyle.TextMargins.Left + oStyle.TextMargins.Right;
			oGraphics.Dispose();

			// Check if we don't have the space
			if (oGrid.Model.ColWidths[nCol] - nMarginSpace < fTextWidth)
				return true;
			return false;
		}

		private void ShowExtendedText(Syncfusion.Windows.Forms.Grid.GridControlBase  oGrid, int nRow, int nCol)
		{
			
			// Set the label text and width
			Graphics oGraphics = oGrid.CreateGraphics();
			GridStyleInfo oStyle = oGrid.Model[nRow, nCol];
            float fTextWidth = oGraphics.MeasureString(oStyle.Text, oStyle.GdipFont).Width;
			if( oStyle.CellType != "ColumnHeaderCell")
			{
				m_oLabel.Text = oStyle.Text;
				m_oLabel.Size = new System.Drawing.Size((int)fTextWidth + 5, oGrid.DefaultRowHeight);
			}
			else if( ExtendedHeaderText.Contains(nCol) ) 
			{
				object ExtHeaderText = ExtendedHeaderText[nCol];
				if( ExtHeaderText != null )
				{
					m_oLabel.Text = ExtHeaderText.ToString();
					fTextWidth = oGraphics.MeasureString(ExtHeaderText.ToString() , oStyle.GdipFont).Width;
					m_oLabel.Size = new System.Drawing.Size((int)fTextWidth + 5, oGrid.DefaultRowHeight);
				}
			}
			
			oGraphics.Dispose();
			
			// Compute the cell location and set the label to that position
			Point oCellPoint = oGrid.ViewLayout.ClientRowColToPoint(nRow, nCol);
			Point oFormPoint = oGrid.Location;
			oFormPoint.X += oCellPoint.X + oStyle.TextMargins.Left;
			oFormPoint.Y += oCellPoint.Y + oStyle.TextMargins.Top;
			m_oLabel.Location = oFormPoint;
			m_oLabel.Visible = true;
			m_oLabel.Font = oGrid.Font;
			m_oLabel.BringToFront();
		}

		private void Label_Click(object sender, System.EventArgs e)
		{
			m_oLabel.Hide();
		}
	}
	#endregion
}

